part of '../flutterx_live_data.dart';

/// Link [LiveData] observers to the lifecycle of your [State]
///
/// Implement this mixin on your widget's [State] to add [Observer]s to a [LiveData] within the state lifecycle
/// Every LiveData can be observed inside [registerObservers] method that you can override
/// The observer must be added through proper extension methods and will be removed automatically on state [dispose]
/// Check [StateObserverExt]
mixin StateObserver<T extends StatefulWidget> on State<T> {
  final List<ObserverWrapper> _disposables = [];
  int _key = -1;

  @override
  void initState() {
    super.initState();
    _doRegister();
  }

  @override
  void didUpdateWidget(T oldWidget) {
    super.didUpdateWidget(oldWidget);
    _doRegister();
  }

  @override
  void dispose() {
    super.dispose();
    for (final disposable in _disposables) disposable.dispose();
  }

  /// Override this method to observe you LiveData(s)
  ///
  /// This is called only once, after the first frame of this widget is scheduled
  void registerObservers(BuildContext context);

  /// Observe multiple sources simultaneously within this context. Check [StateObserverExt]
  void observeAll(Iterable<LiveData> sources, void Function(List values) onChanged) =>
      _observeAllInternal(sources, onChanged, _observe);

  void _doRegister() => Dispatcher.postFrame.post(() {
        _key = 0;
        registerObservers(context);
        while (_key < _disposables.length) _disposables.removeAt(_key++).dispose();
        _key = -1;
      });

  void _observe<V>(LiveData<V> data, Observer<V> onChanged) {
    assert(_key != -1, 'Observe can only be called within registerObservers function');
    final key = _key++;
    if (key < _disposables.length) {
      final existing = _disposables[key];
      if (existing.equals(data, onChanged)) return;
      existing.dispose();
      _disposables[key] = data.observeForever(onChanged);
    } else {
      _disposables.add(data.observeForever(onChanged));
    }
  }
}
